package org.gcn.plinguacore.util.check;

import java.util.List;


public abstract class DecoratorCheck<T> implements InnerCheck<T>{
	
	private static final long serialVersionUID = 339922718160092202L;


	public boolean initializedCauses() {
		// TODO Auto-generated method stub
		return decorated.initializedCauses();
	}

	protected InnerCheck<T> decorated;

	/**
	 * Creates a new DecoratorCheckRule instance, which checks only the restrictions defined on itself.
	 */
	public DecoratorCheck(){
		this(new BaseCheck<T>());
	}
	
	
	/**
	 * Creates a new DecoratorCheckRule instance, which wraps cr as stated
	 * by Decorator pattern. Thus, it will be capable to test both the instance restrictions and cr restrictions 
	 * 
	 * @param cr
	 *            the CheckRule instance to be wrapped
	 */
	public DecoratorCheck(Check<T> cr) {
		if (cr == null)
			throw new NullPointerException(
					"Check argument shouldn't be null");
		this.decorated = (InnerCheck<T>) cr;
	}

	/**
	 * @see org.gcn.plinguacore.util.psystem.rule.checkRule.CheckRule#checkRule(org.gcn.plinguacore.util.psystem.rule.Rule)
	 */
	@Override
	public boolean check(T r) {
		// TODO Auto-generated method stub
		boolean b = checkSpecificPart(r);

		if (!b) {
			if (!initializedCauses()) {
				initCauses();
				getCauses().clear();

			}
			getCauses().add(getSpecificCause());
		}

		return b & decorated.check(r);
	}


	/**
	 * Initializes the simulator message causes
	 */
	public void initCauses() {
		decorated.initCauses();

	}


	/**
	 * Gets a string reporting the causes of the simulator messages
	 * @return a string reporting the causes of the simulator messages
	 * 
	 */
	public String getCausesString() {
		// TODO Auto-generated method stub
		return decorated.getCausesString();
	}

	/**
	 * @see org.gcn.plinguacore.util.psystem.rule.checkRule.CheckRule#getCauses()
	 */
	public List<String> getCauses() {
		return decorated.getCauses();
	}

	protected abstract boolean checkSpecificPart(T r);

	protected abstract String getSpecificCause();


}
