/* 
 * pLinguaCore: A JAVA library for Membrane Computing
 *              http://www.p-lingua.org
 *
 * Copyright (C) 2009  Research Group on Natural Computing
 *                     http://www.gcn.us.es
 *                      
 * This file is part of pLinguaCore.
 *
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * pLinguaCore is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pLinguaCore.  If not, see <http://www.gnu.org/licenses/>.
 */


package org.gcn.plinguacore.util.psystem;

import java.io.Serializable;
import java.util.Collection;

import org.gcn.plinguacore.util.HashMultiSet;
import org.gcn.plinguacore.util.MultiSet;
import org.gcn.plinguacore.util.psystem.membrane.Membrane;



/**
 * An abstract class for a Configuration
 * 
 *  @author Research Group on Natural Computing (http://www.gcn.us.es)
 * 
 */
public abstract class Configuration implements Serializable, Cloneable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6642873032924795615L;



	private MultiSet<String> environment;

	private int number = 0;

	
	private Psystem psystem;
	/**
	 * @param psystem
	 *            A P System
	 */
	public Configuration(Psystem psystem) {
		setPsystem(psystem);
		environment = new HashMultiSet<String>();
	}
	
	/**
	 * Sets the configuration P-system
	 * @param psystem the P-system to be set
	 */
	public void setPsystem(Psystem psystem){
		if (psystem == null)
			throw new NullPointerException("Null P system");
		this.psystem = psystem;
	}

	/**
	 * @return the P System for this configuration
	 */
	public Psystem getPsystem() {
		return psystem;
	}

	/**
	 * @return the number of configuration, the initial one is 0
	 */
	public int getNumber() {
		return number;
	}

	/**
	 * Sets the number of configuration
	 * 
	 * @param number
	 *            the number of the configuration
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/**
	 * @return A Multiset<String> for the environment
	 */
	public MultiSet<String> getEnvironment() {
		return environment;
	}

	/**
	 * @return A Collection<? extends Membrane> of membranes for this
	 *         configuration
	 */
	public abstract Collection<? extends Membrane> getMembranes();

	/**
	 * @see java.lang.Object#clone()
	 */
	@Override
	public abstract Object clone();

	@Override
	public String toString() {
		String environmentString = "";
		if (!this.environment.isEmpty())
			environmentString = environment.toString();
		return environmentString;
	}

}
