/* 
 * pLinguaCore: A JAVA library for Membrane Computing
 *              http://www.p-lingua.org
 *
 * Copyright (C) 2009  Research Group on Natural Computing
 *                     http://www.gcn.us.es
 *                      
 * This file is part of pLinguaCore.
 *
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * pLinguaCore is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pLinguaCore.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.gcn.plinguacore.util.psystem.cellLike;

import java.util.Collection;
import java.util.HashSet;

import org.gcn.plinguacore.util.psystem.Configuration;
import org.gcn.plinguacore.util.psystem.Psystem;
import org.gcn.plinguacore.util.psystem.cellLike.membrane.CellLikeSkinMembrane;
import org.gcn.plinguacore.util.psystem.membrane.Membrane;



/**
 * This class represents configurations in cell-like P-systems
 * 
 *  @author Research Group on Natural Computing (http://www.gcn.us.es)
 * 
 */
public class CellLikeConfiguration extends Configuration implements Cloneable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 5324665255482026462L;
	private CellLikeSkinMembrane skinMembrane = null;

	/**
	 * Creates a new cell-like configuration for a p-system passed as argument
	 * 
	 * @param psystem
	 *            the p-system which this instance represents a configuration to
	 */
	public CellLikeConfiguration(Psystem psystem) {
		super(psystem);
	}

	/**
	 * Gets the skin membrane of the configuration
	 * 
	 * @return the skin membrane of the configuration
	 */
	public CellLikeSkinMembrane getSkinMembrane() {
		return skinMembrane;
	}

	/**
	 * Sets the skin membrane of the configuration
	 * 
	 * @param skinMembrane
	 *            the skin membrane of the configuration
	 */
	public void setSkinMembrane(CellLikeSkinMembrane skinMembrane) {
		this.skinMembrane = skinMembrane;
	}

	/**
	 * Gets all membranes of the current configuration
	 * 
	 * @return a collection containing all membranes of the current
	 *         configuration
	 */
	@Override
	public Collection<? extends Membrane> getMembranes() {
		if (skinMembrane == null)
			return new HashSet<Membrane>();
		return skinMembrane.getAllMembranes();
	}

	/**
	 * @see org.gcn.plinguacore.util.psystem.Configuration#clone()
	 */
	@Override
	public Object clone() {
		CellLikeConfiguration c = new CellLikeConfiguration(
				getPsystem());

		c.setSkinMembrane(getSkinMembrane().clone());
		c.setNumber(getNumber());
		c.getEnvironment().addAll(getEnvironment());
		return c;
	}


}
