/* 
 * pLinguaCore: A JAVA library for Membrane Computing
 *              http://www.p-lingua.org
 *
 * Copyright (C) 2009  Research Group on Natural Computing
 *                     http://www.gcn.us.es
 *                      
 * This file is part of pLinguaCore.
 *
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * pLinguaCore is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pLinguaCore.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.gcn.plinguacore.util.psystem.cellLike;

import java.util.Collection;
import java.util.Iterator;
import java.util.HashSet;

import org.gcn.plinguacore.util.psystem.Configuration;
import org.gcn.plinguacore.util.psystem.Psystem;
import org.gcn.plinguacore.util.psystem.cellLike.membrane.CellLikeMembrane;
import org.gcn.plinguacore.util.psystem.cellLike.membrane.CellLikeNoSkinMembrane;
import org.gcn.plinguacore.util.psystem.cellLike.membrane.CellLikeSkinMembrane;
import org.gcn.plinguacore.util.psystem.membrane.Membrane;



/**
 * This class represents P-systems belonging to cell-like group
 * 
 * @author Research Group on Natural Computing (http://www.gcn.us.es)
 * 
 */
public class CellLikePsystem extends Psystem {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8332316133669389557L;
	private CellLikeSkinMembrane skin = null;

	/**
	 * Creates a new P-system without skin membrane
	 */
	public CellLikePsystem() {
		super();

	}

	/**
	 * Returns the current skin membrane
	 * 
	 * @return the current CellLikeSkinMembrane instance
	 */
	public CellLikeSkinMembrane getSkinMembrane() {
		return skin;
	}

	/**
	 * Sets an CellLikeSkinMembrane instance as the current skin membrane
	 * 
	 * @param skinMembrane
	 *            the CellLikeSkinMembrane instance to set
	 */
	public void setSkinMembrane(CellLikeSkinMembrane skinMembrane) {
		this.skin = skinMembrane;
	}

	/*
	 * IPHM Funcion que añade los multisets iniciales a la configuracion
	 * inicial
	 */
	private void addInitialMultiSets(CellLikeMembrane m) {
		/*
		 * If there's any initial multiset which corresponds to the membrane
		 * label, it should be added into the membrane
		 */
		if (getInitialMultiSets().containsKey(m.getLabel()))
			m.getMultiSet().addAll(getInitialMultiSets().get(m.getLabel()));
		/* Then, its alphabet objects should be added */
		addAlphabetObjects(m.getMultiSet());
		/* Finally, each child is added its initial multiset (in case it exists) */
		Iterator<CellLikeNoSkinMembrane> it = m.getChildMembranes().iterator();
		while (it.hasNext())
			addInitialMultiSets(it.next());
	}

	/**
	 * 
	 * @see org.gcn.plinguacore.util.psystem.Psystem#getFirstConfiguration()
	 */
	@Override
	public Configuration getFirstConfiguration() {
		CellLikeConfiguration config = new CellLikeConfiguration(this);
		config.setNumber(0);
		config.setSkinMembrane(skin.clone());
		addInitialMultiSets(config.getSkinMembrane());
		return config;
	}

	/**
	 * @see org.gcn.plinguacore.util.psystem.Psystem#toString()
	 */
	@Override
	public String toString() {
		String str = "";
		if (skin != null)
			str = skin.toString();
		return str + "\n\n" + super.toString();
	}

	/**
	 * Returns all P-system membranes
	 * 
	 * @return a collection containing all P-system membranes
	 */
	@Override
	public Collection<? extends Membrane> getMembranes() {
		if (skin == null)
			return new HashSet<CellLikeMembrane>();
		return skin.getAllMembranes();
	}
}
